from .service_account import ServiceAccount
from .jwt_handler import JWTHandler


class TokenRefresher:
    _jwt_handler: JWTHandler
    _token: str

    def __init__(self, service_account: ServiceAccount, seconds_before_expire: int, audience: str,
                 token_duration: int) -> None:
        self._jwt_handler = JWTHandler(service_account, seconds_before_expire, audience, token_duration)
        self._token = None

    def get_access_token(self):
        if self._token is None or self._jwt_handler.is_token_expired(self._token):
            self._token = self._jwt_handler.generate_token()

        return self._token



import json
from dataclasses import dataclass, fields


@dataclass
class ServiceAccount:
    client_email: str
    client_id: str
    private_key: str
    private_key_id: str

    @staticmethod
    def from_file(path: str) -> 'ServiceAccount':

        def filter_fields(d: dict):
            field_names = {f.name for f in fields(ServiceAccount)}
            return {k: d[k] for k in d if k in field_names}

        with open(path, 'r') as f:
            return json.load(f, object_hook=lambda d: ServiceAccount(**filter_fields(d)))



import re


def get_next_page_url(response_headers: dict) -> str:
    link_header = response_headers.get('Link', "")

    match = re.findall("<([^<>]+)>;rel=\"([^\"]+)\"", link_header)

    for link, rel in match:
        if rel == 'next':
            return link

    return None



from setuptools import setup

setup(
    install_requires=[
        "cffi==1.15.1",
        "cryptography==40.0.2",
        "dataclasses==0.6; python_version == '3.6'",
        "pycparser==2.21",
        "pyjwt[crypto]==2.7.0",
    ],
    dependency_links=[],
)



import datetime
import jwt

from .service_account import ServiceAccount

JWT_ALGORITHM = 'RS256'

MAX_EXPIRATION_DELTA = 60 * 60 * 24 * 7  # 7 days
MIN_EXPIRATION_DELTA = 60 * 60  # 1 hour


class JWTHandler:
    _service_account: ServiceAccount
    _audience: str
    _seconds_before_expire: int
    _expiration_time: datetime.timedelta

    def __init__(self, service_account: ServiceAccount, seconds_before_expire: int, audience: str,
                 token_duration: int) -> None:

        if token_duration < MIN_EXPIRATION_DELTA or token_duration > MAX_EXPIRATION_DELTA:
            raise ValueError(f'token_duration should be greater than or equal to {MIN_EXPIRATION_DELTA} and less than'
                             + ' or equal to {MAX_EXPIRATION_DELTA}')
        self._service_account = service_account
        self._audience = audience
        self._expiration_time = datetime.timedelta(seconds=token_duration)
        self._seconds_before_expire = seconds_before_expire

    def generate_token(self) -> str:
        now = datetime.datetime.utcnow()

        return jwt.encode(
            {
                'iss': self._service_account.client_email,
                'sub': self._service_account.client_email,
                'aud': self._audience,
                'iat': now,
                'exp': now + self._expiration_time,
            },
            self._service_account.private_key,
            algorithm=JWT_ALGORITHM,
            headers={
                'kid': self._service_account.private_key_id,
                'typ': 'JWT',
                'alg': JWT_ALGORITHM,
            })

    def is_token_expired(self, token: str) -> bool:
        decoded = jwt.decode(token,
                             self._service_account.private_key,
                             algorithms=JWT_ALGORITHM,
                             options={"verify_signature": False})
        exp_time = datetime.datetime.utcfromtimestamp(decoded['exp'])

        if exp_time - datetime.datetime.utcnow() < datetime.timedelta(seconds=self._seconds_before_expire):
            return True
        return False



from .service_account import ServiceAccount
from .token_refresher import TokenRefresher

DEFAULT_AUDIENCE = 'https://api.combocurve.com'


class ComboCurveAuth:
    _token_refresher: TokenRefresher
    _api_key: str

    def __init__(self,
                 service_account: ServiceAccount,
                 api_key: str,
                 seconds_before_token_expire: int = 60,
                 audience: str = DEFAULT_AUDIENCE,
                 token_duration=3600) -> None:
        """
        Parameters:
            service_account (ServiceAccount): ComboCurve Service Account
            api_key (str): ComboCurve API key
            seconds_before_token_expire (int): How many seconds before the token expiration to generate a new token.
            audience (str): What the token is intended for.
            token_duration (int): How many seconds the token is going to be valid.
        """
        self._token_refresher = TokenRefresher(service_account, seconds_before_token_expire, audience, token_duration)
        self._api_key = api_key

    def get_auth_headers(self):
        token = self._token_refresher.get_access_token()
        return {'Authorization': f'Bearer {token}', 'x-api-key': self._api_key}
